import React from 'react';
import ReactDOM from 'react-dom';
import './index.css';

//每个宫格的<button>组件
// 当组件不包含state时，可以使用函数组件
function Square(props) {
  return (
    <button
      className="square"
      onClick={() => props.onClick()}
    >
      {/*Board把点击事件传递给Square， 当button的点击事件触发时，就是触发了Board的点击事件 */}
      {/* 显示父组件Board传入的值 */}
      {props.value}
    </button>
  );
}

// 九宫格框架组件
class Board extends React.Component {

  renderSquare(i) {
    // 渲染Square组件 
    // 从父组件game中获取数据
    return (
      <Square
        value={this.props.squares[i]}
        onClick={() => this.props.onClick(i)} />
    )
  }

  render() {
    return (
      <div>
        {/* 提示下一步的执行者 */}
        {/* <div className="status">{status}</div> */}
        <div className="board-row">
          {this.renderSquare(0)}
          {this.renderSquare(1)}
          {this.renderSquare(2)}
        </div>
        <div className="board-row">
          {this.renderSquare(3)}
          {this.renderSquare(4)}
          {this.renderSquare(5)}
        </div>
        <div className="board-row">
          {this.renderSquare(6)}
          {this.renderSquare(7)}
          {this.renderSquare(8)}
        </div>
      </div>
    );
  }
}



// 渲染棋盘
class Game extends React.Component {
  // 进行状态提升（即父组件向子组件传值）
  constructor(props) {
    super(props);
    this.state = {
      history: [{
        squares: Array(9).fill(null),
      }],
      xIsNext: true,
      // 正在查看哪一项的历史记录
      stepNumber: 0,
    }
  }
  handleClick(i) {
    const history = this.state.history.slice(0, this.state.stepNumber + 1);
    const current = history[history.length - 1];
    // 当点击时更新按钮的内容
    // slice方法返回一个新的数组，当没有参数时，返回所有的
    const squares = current.squares.slice();
    // 当有玩家胜出时/Square已经被填充时，该函数直接返回
    if (calculateWinner(squares) || squares[i]) {
      return
    }
    squares[i] = this.state.xIsNext ? 'X' : 'O';
    this.setState({
      // 把新的历史记录拼接到history上，concat不会改变原来的数组
      history: history.concat([{
        squares: squares,
      }]),
      stepNumber: history.length,
      xIsNext: !this.state.xIsNext,
    });
  }
  jumpTo(step) {
    this.setState({
      stepNumber: step,
      // step为偶数时，xIsNext设为true
      xIsNext: (step % 2) === 0
    })
  }
  render() {
    const history = this.state.history;
    // 根据stepNumber渲染
    const current = history[this.state.stepNumber];
    const winner = calculateWinner(current.squares)
    const moves = history.map((step, move) => {
      const desc = move ? 'Go to move #' + move : 'Go to game start';
      return (
        <li key={move}>
          <button onClick={() => this.jumpTo(move)}>{desc}</button>
        </li>
      )
    })
    let status;
    if (winner) {
      status = '获胜者:' + winner;
    } else {
      status = 'Next player:' + (this.state.xIsNext ? 'X' : 'O');
    }
    return (
      <div className="game">
        <div className="game-board">
          {/* 渲染Board组件 */}
          <Board
            squares={current.squares}
            onClick={(i) => this.handleClick(i)}
          />
        </div>
        <div className="game-info">
          <div>{status}</div>
          <ol>{moves}</ol>
        </div>
      </div>
    );
  }
}

// ========================================

ReactDOM.render(
  <Game />,
  document.getElementById('root')
);


// 判断哪个玩家胜出
function calculateWinner(squares) {
  // 胜出的线路
  const lines = [
    [0, 1, 2],
    [3, 4, 5],
    [6, 7, 8],
    [0, 3, 6],
    [1, 4, 7],
    [2, 5, 8],
    [0, 4, 8],
    [2, 4, 6],
  ];
  for (let i = 0; i < lines.length; i++) {
    const [a, b, c] = lines[i];
    if (squares[a] && squares[a] === squares[b] && squares[a] === squares[c]) {
      return squares[a];
    }
  }
  return null;
}